unit lclfpreportbuild;

{
  This is a unit just to force the IDE to build all fpreport units
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  fpreporthtmlutil,
  fpreporthtmlparser,
  fpreportstreamer,
  fpreport,
  fpreportdb,
  fpreportcanvashelper,
  fpreportcontnr,
  fpreportbarcode,
  fpreportqrcode,
  fpreportdatacsv,
  fpreportdatadbf,
  fpreportdatajson,
  fpreportdata,
  fpreportjson,
  fpreportdatasqldb,
  fpreportfpimageexport,
  fpreporthtmlexport,
  fpreportpdfexport;

implementation

end.

