..\..\..\tools\lazres ..\..\cursors.res cur_1.cur cur_10.cur cur_12.cur cur_13.cur cur_14.cur cur_15.cur cur_16.cur cur_17.cur cur_18.cur cur_20.cur cur_21.cur cur_22.cur
..\..\..\tools\lazres ..\..\cursor_hourglass.res cur_11.cur