unit reggoogleapi;

interface

{$mode objfpc}{$h+}

uses sysutils,classes;

procedure register;

implementation

uses
  restbase,
  googleservice,
  googlebase,
  googleclient,
  googleadexchangebuyer,
  googleadexchangeseller,
  googleadmin,
  googleadsense,
  googleadsensehost,
  googleanalytics,
  googleandroidenterprise,
  googleandroidpublisher,
  googleappsactivity,
  googleappstate,
  googleaudit,
  googleautoscaler,
  googlebigquery,
  googleblogger,
  googlebooks,
  googlecalendar,
  googlecivicinfo,
  googlecloudmonitoring,
  googlecompute,
  googlecomputeaccounts,
  googlecontainer,
  googlecontent,
  googlecoordinate,
  googlecustomsearch,
  googledataflow,
  googledatastore,
  googledeploymentmanager,
  googledfareporting,
  googlediscovery,
  googledns,
  googledoubleclickbidmanager,
  googledoubleclicksearch,
  googledrive,
  googlefitness,
  googlefreebase,
  googlefusiontables,
  googlegames,
  googlegamesConfiguration,
  googlegamesManagement,
  googlegan,
  googlegenomics,
  googlegmail,
  googlegroupsmigration,
  googlegroupssettings,
  googleidentitytoolkit,
  googlelicensing,
  googlemanager,
  googlemapsengine,
  googlemirror,
  googleoauth2,
  googlepagespeedonline,
  googleplus,
  googleplusDomains,
  googleprediction,
  googlepubsub,
  googleqpxExpress,
  googlereplicapool,
  googlereplicapoolupdater,
  googlereseller,
  googleresourceviews,
  googlesiteVerification,
  googlespectrum,
  googlesqladmin,
  googlestorage,
  googletagmanager,
  googletaskqueue,
  googletasks,
  googletranslate,
  googleurlshortener,
  googlewebfonts,
  googlewebmasters,
  googleyoutube,
  googleyoutubeAnalytics,
  googlecloudlatencytest,
  googlecloudsearch,
  googlelogging;


procedure register;

begin
  RegisterComponents('Google API',[
    TGoogleClient,
    TAdexchangebuyerAPI,
    TAdexchangesellerAPI,
    TAdminAPI,
    TAdsenseAPI,
    TAdsensehostAPI,
    TAnalyticsAPI,
    TAndroidenterpriseAPI,
    TAndroidpublisherAPI,
    TAppsactivityAPI,
    TAppstateAPI,
    TAuditAPI,
    TAutoscalerAPI,
    TBigqueryAPI,
    TBloggerAPI,
    TBooksAPI,
    TCalendarAPI,
    TCivicinfoAPI,
    TCloudmonitoringAPI,
    TComputeAPI,
    TComputeaccountsAPI,
    TContainerAPI,
    TContentAPI,
    TCoordinateAPI,
    TCustomsearchAPI,
    TDataflowAPI,
    TDatastoreAPI,
    TDeploymentmanagerAPI,
    TDfareportingAPI,
    TDiscoveryAPI,
    TDnsAPI,
    TDoubleclickbidmanagerAPI,
    TDoubleclicksearchAPI,
    TDriveAPI,
    TFitnessAPI,
    TFreebaseAPI,
    TFusiontablesAPI,
    TGamesAPI,
    TGamesConfigurationAPI,
    TGamesManagementAPI,
    TGanAPI,
    TGenomicsAPI,
    TGmailAPI,
    TGroupsmigrationAPI,
    TGroupssettingsAPI,
    TIdentitytoolkitAPI,
    TLicensingAPI,
    TManagerAPI,
    TMapsengineAPI,
    TMirrorAPI,
    TOauth2API,
    TPagespeedonlineAPI,
    TPlusAPI,
    TPlusDomainsAPI,
    TPredictionAPI,
    TPubsubAPI,
    TQpxExpressAPI,
    TReplicapoolAPI,
    TReplicapoolupdaterAPI,
    TResellerAPI,
    TResourceviewsAPI,
    TSiteVerificationAPI,
    TSpectrumAPI,
    TSqladminAPI,
    TStorageAPI,
    TTagmanagerAPI,
    TTaskqueueAPI,
    TTasksAPI,
    TTranslateAPI,
    TUrlshortenerAPI,
    TWebfontsAPI,
    TWebmastersAPI,
    TYoutubeAPI,
    TYoutubeAnalyticsAPI,
    TCloudlatencytestAPI,
    TCloudsearchAPI,
    TLoggingAPI
   ]);
end;

end.
