unit IconFinderStrConstsIDE;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

resourcestring

  // IconFinderFrm
  RSIconFinderIDE_Caption = 'Icon Finder (%d out of %d icons)';
  RSIconFinderIDE_Select = 'Select';

  // IconFinderSettings
  RSIconFinderIDE_IconLibrary = 'Icon Finder';
  RSIconFinderIDE_Folders = 'Folders ...';
  RSIconFinderIDE_FolderHint = 'Icon folder management';
  RSIconFinderIDE_Metadata = 'Metadata ...';
  RSIconFinderIDE_MetadataHint = 'Edit icon metadata';
  RSIconFinderIDE_SaveMetadata = 'Save metadata';
  RSIconFinderIDE_SaveMetadataHint = 'Save icon metadata files';
  RSIconFinderIDE_General = 'General';

  // ImageListEditorEx
  RSImgListEditor_AddFromIconFinder = 'Add from Icon Finder ...';
  RSImgListEditor_ReplaceFromIconFinder = 'Replace by Icon Finder ...';

  // GraphicPropEditEx
  RSGraphPropEditor_IconFinder = 'Icon Finder ...';

implementation

end.

