#define RT_MANIFEST  24
#define CREATEPROCESS_MANIFEST_RESOURCE_ID 1
#define ISOLATIONAWARE_MANIFEST_RESOURCE_ID 2
#define ISOLATIONAWARE_NOSTATICIMPORT_MANIFEST_RESOURCE_ID 3

CREATEPROCESS_MANIFEST_RESOURCE_ID RT_MANIFEST MOVEABLE PURE
{
 "<?xml version=""1.0"" encoding=""UTF-8"" standalone=""yes""?>"
 "<assembly xmlns=""urn:schemas-microsoft-com:asm.v1"" manifestVersion=""1.0"">"
 "<assemblyIdentity version=""1.0.0.0"" processorArchitecture=""*"" name=""CompanyName.ProductName.YourApp"" type=""win32""/>"
 "<description>Your application description here.</description>"
 "<dependency>"
 "<dependentAssembly>"
 "<assemblyIdentity type=""win32"" name=""Microsoft.Windows.Common-Controls"" version=""6.0.0.0"" processorArchitecture=""*"" publicKeyToken=""6595b64144ccf1df"" language=""*""/>"
 "</dependentAssembly>"
 "</dependency>"
 "<trustInfo xmlns=""urn:schemas-microsoft-com:asm.v3"">"
 "<security>"
 "<requestedPrivileges>"
 "<requestedExecutionLevel level=""asInvoker"" uiAccess=""false""/>"
 "</requestedPrivileges>"
 "</security>"
 "</trustInfo>"
 "</assembly>"
}