{ This file was automatically created by Lazarus. Do not edit!
  This source is only used to compile and install the package.
 }

unit fpvectorialpkg;

{$warn 5023 off : no warning about unused units}
interface

uses
  fpvectorial, fpvtocanvas, fpvectbuildunit, fpvutils, avisozlib, 
  avisocncgcodewriter, avisocncgcodereader, cdrvectorialreader, 
  dxfvectorialreader, epsvectorialreader, lasvectorialreader, 
  lazvectorialreader, mathmlvectorialreader, odgvectorialreader, 
  rawvectorialreadwrite, svgvectorialreader_rsvg, svgvectorialwriter, 
  svgzvectorialreader, odtvectorialwriter, docxvectorialwriter, 
  htmlvectorialreader, svgvectorialreader, fpvWMF, wmfvectorialreader, 
  wmfvectorialwriter, fpvectorial2canvas, pdfvectorialwriter;

implementation

end.
