NavArrow_F     RCDATA "../../../images/general_purpose/Arrow_41_16.png"
NavArrow_F_150 RCDATA "../../../images/general_purpose/Arrow_41_24.png"
NavArrow_F_200 RCDATA "../../../images/general_purpose/Arrow_41_32.png"
NavArrow_L     RCDATA "../../../images/general_purpose/Arrow_37_16.png"
NavArrow_L_150 RCDATA "../../../images/general_purpose/Arrow_37_24.png"
NavArrow_L_200 RCDATA "../../../images/general_purpose/Arrow_37_32.png"
NavArrow_R     RCDATA "../../../images/general_purpose/Arrow_36_16.png"
NavArrow_R_150 RCDATA "../../../images/general_purpose/Arrow_36_24.png"
NavArrow_R_200 RCDATA "../../../images/general_purpose/Arrow_36_32.png"
NavArrow_E     RCDATA "../../../images/general_purpose/Arrow_40_16.png"
NavArrow_E_150 RCDATA "../../../images/general_purpose/Arrow_40_24.png"
NavArrow_E_200 RCDATA "../../../images/general_purpose/Arrow_40_32.png"
NavPlus     RCDATA "../../../images/general_purpose/Add_07_16.png"
NavPlus_150 RCDATA "../../../images/general_purpose/Add_07_24.png"
NavPlus_200 RCDATA "../../../images/general_purpose/Add_07_32.png"
NavMinus     RCDATA "../../../images/general_purpose/Remove_07_16.png"
NavMinus_150 RCDATA "../../../images/general_purpose/Remove_07_24.png"
NavMinus_200 RCDATA "../../../images/general_purpose/Remove_07_32.png"
NavArrow_Hide     RCDATA "../../../images/general_purpose/Arrow_07_16.png"
NavArrow_Hide_150 RCDATA "../../../images/general_purpose/Arrow_07_24.png"
NavArrow_Hide_200 RCDATA "../../../images/general_purpose/Arrow_07_32.png"
NavArrow_Show     RCDATA "../../../images/general_purpose/Arrow_08_16.png"
NavArrow_Show_150 RCDATA "../../../images/general_purpose/Arrow_08_24.png"
NavArrow_Show_200 RCDATA "../../../images/general_purpose/Arrow_08_32.png"
NavArrow_T     RCDATA "../../../images/general_purpose/Arrow_42_16.png"
NavArrow_T_150 RCDATA "../../../images/general_purpose/Arrow_42_24.png"
NavArrow_T_200 RCDATA "../../../images/general_purpose/Arrow_42_32.png"
NavArrow_U     RCDATA "../../../images/general_purpose/Arrow_38_16.png"
NavArrow_U_150 RCDATA "../../../images/general_purpose/Arrow_38_24.png"
NavArrow_U_200 RCDATA "../../../images/general_purpose/Arrow_38_32.png"
NavArrow_D     RCDATA "../../../images/general_purpose/Arrow_39_16.png"
NavArrow_D_150 RCDATA "../../../images/general_purpose/Arrow_39_24.png"
NavArrow_D_200 RCDATA "../../../images/general_purpose/Arrow_39_32.png"
NavArrow_B     RCDATA "../../../images/general_purpose/Arrow_43_16.png"
NavArrow_B_150 RCDATA "../../../images/general_purpose/Arrow_43_24.png"
NavArrow_B_200 RCDATA "../../../images/general_purpose/Arrow_43_32.png"
