unit SynDesignStringConstants;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

resourcestring
  syndsLineNumbers = 'Line Numbers';
  syndsCodeFolding = 'Code Folding';
  syndsChangeMarker = 'Change Marker';
  syndsBookmarks = 'Bookmarks';
  syndsSeparator = 'Separator';
  syndsLineOverview = 'Line Overview';

  syndsResetMouseActions = 'Reset mouse actions';


implementation

end.

