
function KillTimer(hWnd: TLCLHandle; nIDEvent: LongWord): Boolean;
begin
end;

function SetTimer(hWnd: TLCLHandle; nIDEvent: LongWord; uElapse: LongWord; lpTimerFunc: TTimerNotify): LongWord;
begin
end;
