unit tbar;
{$mode ObjFPC}{$H+}
interface
type
  bar = (xb1, xb2);
  abc = (xa1, xa2);

  unit1 = (tbar_unit1_a, tbar_unit1_b);
  project1 = (tbar_project1_a, tbar_project1_b);
  tfoo = (tbar_tfoo_a, tbar_tfoo_b);

implementation

end.

