{ This file was automatically created by Lazarus. Do not edit!
  This source is only used to compile and install the package.
 }

unit IconFinder_pkg;

{$warn 5023 off : no warning about unused units}
interface

uses
  BasicThumbnails, IconKeywordFilterEditor, IconThumbNails, IconViewer, 
  IconFinderFolders, IconFinderMetaData, IconFinderStrConsts;

implementation

end.
