{$include components/lazutils/fpmake.pp}
{$include packager/registration/fpmake.pp}
{$include lcl/fpmake.pp}
{$include lcl/interfaces/fpmake.pp}
{$include components/lazcontrols/fpmake.pp}
{$include components/codetools/fpmake.pp}
{$include components/ideintf/fpmake.pp}
{$include components/synedit/fpmake.pp}
{$include components/debuggerintf/fpmake.pp}
{$include components/lazdebuggergdbmi/fpmake.pp}
{$include ide/fpmake.pp}

