unit DbgIntfCommonStrings;

{$mode objfpc}{$H+}

interface

resourcestring
  rsRunErrorInvalidFunctionNumber = 'Invalid function number';
  rsRunErrorFileNotFound = 'File not found';
  rsRunErrorPathNotFound = 'Path not found';
  rsRunErrorTooManyOpenFiles = 'Too many open files';
  rsRunErrorFileAccessDenied = 'File access denied';
  rsRunErrorInvalidFileHandle = 'Invalid file handle';
  rsRunErrorInvalidFileAccessCode = 'Invalid file access code';
  rsRunErrorInvalidDriveNumber = 'Invalid drive number';
  rsRunErrorCannotRemoveCurrentDirect = 'Cannot remove current directory';
  rsRunErrorCannotRenameAcrossDrives = 'Cannot rename across drives';
  rsRunErrorDiskReadError = 'Disk read error';
  rsRunErrorDiskWriteError = 'Disk write error';
  rsRunErrorFileNotAssigned = 'File not assigned';
  rsRunErrorFileNotOpen = 'File not open';
  rsRunErrorFileNotOpenForInput = 'File not open for input';
  rsRunErrorFileNotOpenForOutput = 'File not open for output';
  rsRunErrorInvalidNumericFormat = 'Invalid numeric format';
  rsRunErrorInvalidEnumeration = 'Invalid enumeration';
  rsRunErrorDiskIsWriteProtected = 'Disk is write-protected';
  rsRunErrorBadDriveRequestStructLeng = 'Bad drive request struct length';
  rsRunErrorDriveNotReady = 'Drive not ready';
  rsRunErrorCRCErrorInData = 'CRC error in data';
  rsRunErrorDiskSeekError = 'Disk seek error';
  rsRunErrorUnknownMediaType = 'Unknown media type';
  rsRunErrorSectorNotFound = 'Sector Not Found';
  rsRunErrorPrinterOutOfPaper = 'Printer out of paper';
  rsRunErrorDeviceWriteFault = 'Device write fault';
  rsRunErrorDeviceReadFault = 'Device read fault';
  rsRunErrorHardwareFailure = 'Hardware failure';
  rsRunErrorDivisionByZero = 'Division by zero';
  rsRunErrorRangeCheckError = 'Range check error';
  rsRunErrorStackOverflowError = 'Stack overflow error';
  rsRunErrorHeapOverflowError = 'Heap overflow error';
  rsRunErrorInvalidPointerOperation = 'Invalid pointer operation';
  rsRunErrorFloatingPointOverflow = 'Floating point overflow';
  rsRunErrorFloatingPointUnderflow = 'Floating point underflow';
  rsRunErrorInvalidFloatingPointOpera = 'Invalid floating point operation';
  rsRunErrorObjectNotInitialized = 'Object not initialized';
  rsRunErrorCallToAbstractMethod = 'Call to abstract method';
  rsRunErrorStreamRegistrationError = 'Stream registration error';
  rsRunErrorCollectionIndexOutOfRange = 'Collection index out of range';
  rsRunErrorCollectionOverflowError = 'Collection overflow error';
  rsRunErrorArithmeticOverflowError = 'Arithmetic overflow error';
  rsRunErrorGeneralProtectionFault = 'General Protection fault';
  rsRunErrorUnhandledExceptionOccurre = 'Unhandled exception occurred';
  rsRunErrorInvalidValueSpecified = 'Invalid value specified';
  rsRunErrorInvalidTypecast = 'Invalid typecast';
  rsRunErrorVariantDispatchError = 'Variant dispatch error';
  rsRunErrorVariantArrayCreate = 'Variant array create';
  rsRunErrorVariantIsNotAnArray = 'Variant is not an array';
  rsRunErrorVarArrayBoundsCheckError = 'Var Array Bounds check error';
  rsRunErrorAssertionFailedError = 'Assertion failed error';
  rsRunErrorSafecallErrorCheck = 'Safecall error check';
  rsRunErrorExceptionStackCorrupted = 'Exception stack corrupted';
  rsRunErrorThreadsNotSupported = 'Threads not supported';

implementation

end.

