{ This unit collects all resource strings of the project }
unit StringsUnit;

{$mode objfpc}{$H+}

interface

resourcestring
  rsBeer = 'Beer';
  rsWine = 'Wine';
  rsWater = 'Water';
  rsYouSelected = 'You selected %s.';
  rsYouSelectedAt = 'You selected %0:s at %1:s.';
  rsTodayIs = 'Today is %s';
  rsOne = 'One';
  rsTwo = 'Two';
  rsThree = 'Three';
  rsFour = 'Four';
  rsFive = 'Five';
  rsSix = 'Six';
  rsSeven = 'Seven';
  rsEight = 'Eight';
  rsNine = 'Nine';
  rsSumOfSelectedNumbers = 'Sum of selected numbers: %s';

implementation

end.

