ColorSchemeDefault RCDATA "ColorDefault.xml"
ColorSchemeDelphi  RCDATA "ColorDelphi.xml"
ColorSchemeOcean RCDATA "ColorOcean.xml"
ColorSchemePascalClassic RCDATA "ColorPascalClassic.xml"
ColorSchemeTwilight RCDATA "ColorTwilight.xml"
codetemplates RCDATA "codetemplates.dci"