unit activexstrconsts;

{$mode objfpc}{$H+}

interface

resourcestring
  axImportTypeLibrary = 'Import Type Library';
  axSelectDirectoryToStorePackagePLpk = 'Select directory to store package %sP.lpk';
  axFileContainingTypeLibrary = 'File containing type library:';
  axCreateVisualComponentTActiveXContainerDescendant = 'Create visual '
    +'component (TActiveXContainer descendant)';
  axCreatePackage = 'Create package';
  axConvertDependantTypelibs = 'Convert dependant typelibs';
  axTypeLibraryFilesTlbDllExeOcxOlbTlbDllExeOcxOlbAllF = 'Type library files ('
    +'*.tlb;*.dll;*.exe;*.ocx;*.olb)|*.tlb;*.dll;*.exe;*.ocx;*.olb|All Files ('
    +'*.*)|*.*';
  axImportTypeLibraryMenu = 'Import Type Library ...';

implementation

end.

