{ This file was automatically created by Lazarus. Do not edit!
  This source is only used to compile and install the package.
 }

unit IconFinder_dsgn_pkg;

{$warn 5023 off : no warning about unused units}
interface

uses
  ImageListEditorEx, IconFinderSettings, IconFinderReg, IconFinderFrm, 
  GraphPropEditsEx, GraphicPropEditEx, IconFinderCommon, IconFinderStrConstsIDE, 
  LazarusPackageIntf;

implementation

procedure Register;
begin
  RegisterUnit('IconFinderReg', @IconFinderReg.Register);
end;

initialization
  RegisterPackage('iconfinder_dsgn_pkg', @Register);
end.
