/*
  fpdoc.css
  Modified for use with Lazarus LCL and LazUtils documentation
*/

body {
  background-color: white;
  color: black;
  font-family: "Roboto", "DejaVu Sans", "Arial", Helvetica, sans-serif;
  font-size: 11pt;
}

tt, span.kw, pre {
  font-family: "Courier New", Courier, monospace;
  font-size: 11pt;
}

a:link {
  color: blue;
}

a:visited {
  color: darkblue;
}

a:active {
  color: red;
}

a {
  text-decoration: none;
}

a:hover {
  text-decoration: underline;
}

h1, h2, td.h2 {
  color: #005A9C;
}

/* Especially for Netscape on Linux: */
h3, td.h3 {
  font-size: 11pt;
}

/* source fragments */
span.code {
  white-space: nowrap;
  font-weight: bold;
  /* font-size: 1em; */
}

/* symbols in source fragments */
span.sym {
  color: darkred;
}

/* keywords in source fragments */
span.kw {
  font-weight: bold;
}

/* comments in source fragments */
span.cmt {
  color: darkcyan;
  font-style: italic;
}

/* directives in source fragments */
span.dir {
  color: OrangeRed;
  font-style: italic;
}

/* numbers in source fragments */
span.num {
  color: darkmagenta;
}

/* characters (#...) in source fragments */
span.chr {
  color: darkcyan;
}

/* strings in source fragments */
span.str {
  color: blue;
}

/* assembler passages in source fragments */
span.asm {
  color: green;
}

td {
  vertical-align: baseline;
}

td.pre {
  white-space: pre;
}

table caption {
  font-weight: bold;
}

p.cmt {
  color: gray;
}

span.warning {
  color: red;
  font-weight: bold;
}

span.file {
  color: darkgreen;
}

table.remark {
  background-color: #ffffc0;
}

table.bar {
  background-color: #a0c0ff;
}

table.bar tr td {
  font-size: .875em;
}

span.bartitle {
  font-style: normal;
  font-weight: bold;
  color: darkblue;
}

span.footer {
  font-style: italic;
  color: darkblue;
}

/* definition list */
dl {
 display: block; 
}

/* definition list: term */
dt {
 font-weight: bold;
 color: darkgreen;
}

/* definition list: description */
dd { 
 margin-left: 1.5em;
 padding-bottom: .3em;
}

/* for browsers in standards compliance mode */
td p { margin: 0px; }

var { font-weight: bold; }

/* used in custom page footers */
hr.footer-sep { margin-top: .667em; }

table.footer {
  width: 100%;
  margin-top: .5em;
  font-style: normal;
  font-size: .875em;
  color: Black;
}

td.footer-doc { width: 42%; text-align: left; font-weight: bold; }
td.footer-ver { width: 33%; text-align: center; color: Gray; }
td.footer-date { width: 25%; text-align: right; color: Gray; }

/* used on the class hierarchy page */
span.toggletreeclose {
  background: url(minus.png) center left no-repeat;
  padding-left: 20px;
}

span.toggletreeopen {
  background: url(plus.png) center left no-repeat;
  padding-left: 20px;
}

li.classtree ul { display: block; }
ul.classtreelist { list-style-type: none; }
ul.classtreelist li { padding-left: 0px; }
li.classtreeclosed ul { display: none; }
